/**
 * Copyright (c) 2002-2011 "Neo Technology,"
 * Network Engine for Objects in Lund AB [http://neotechnology.com]
 *
 * This file is part of Neo4j.
 *
 * Neo4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
package org.neo4j.onlinebackup.net;

import java.io.IOException;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;

public class AcceptJob extends Job
{
    private final ServerSocketChannel serverChannel;
    
    private SocketChannel acceptedChannel = null;
    
    public AcceptJob( Callback callback, ServerSocketChannel channel )
    {
        super( callback );
        this.serverChannel = channel;
    }
    
    public SocketChannel getAcceptedChannel()
    {
        return acceptedChannel; 
    }
    
    @Override
    public boolean performJob()
    {
        try
        {
            if ( serverChannel.isOpen() )
            {
                acceptedChannel = serverChannel.accept();
            }
            else
            {
                setNoRequeue();
            }
        }
        catch ( IOException e )
        {
            throw new SocketException( 
                "Unable to accept incoming connection", e );
        }
        if ( acceptedChannel != null )
        {
            executeCallback();
            acceptedChannel = null;
            return true;
        }
        return false;
    }
}